"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationRecord = exports.Evaluation = void 0;
const css_select_1 = __importDefault(require("css-select"));
const crypto_1 = require("crypto");
const util_1 = require("@qualweb/util");
const evaluationRecord_object_1 = __importDefault(require("./evaluationRecord.object"));
exports.EvaluationRecord = evaluationRecord_object_1.default;
const wappalyzer_1 = require("@qualweb/wappalyzer");
const endpoint = 'http://194.117.20.242/validate/';
class Evaluation {
    async getEvaluator(page, sourceHtml, url) {
        const [plainHtml, pageTitle, elements, browserUserAgent] = await Promise.all([
            page.evaluate(() => {
                return document.documentElement.outerHTML;
            }),
            page.title(),
            page.$$('*'),
            page.browser().userAgent()
        ]);
        let urlStructure = undefined;
        if (url) {
            urlStructure = this.parseUrl(url, page.url() !== 'about:blank' ? page.url() : url);
        }
        const processedHtml = {
            html: {
                plain: plainHtml
            },
            title: pageTitle,
            elementCount: elements.length
        };
        const viewport = page.viewport();
        const evaluator = {
            name: 'QualWeb',
            description: 'QualWeb is an automatic accessibility evaluator for webpages.',
            version: '3.0.0',
            homepage: 'http://www.qualweb.di.fc.ul.pt/',
            date: new Date().toISOString().replace(/T/, ' ').replace(/\..+/, ''),
            hash: crypto_1.randomBytes(40).toString('hex'),
            url: urlStructure,
            page: {
                viewport: {
                    mobile: !!viewport.isMobile,
                    landscape: !!viewport.isLandscape,
                    userAgent: browserUserAgent,
                    resolution: {
                        width: viewport.width,
                        height: viewport.height
                    }
                },
                dom: {
                    source: sourceHtml,
                    processed: processedHtml
                }
            }
        };
        return evaluator;
    }
    async addQWPage(page) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/qw-page').replace('index.js', 'qwPage.js')
        });
        await page.evaluate(() => {
            window.page = new QWPage.QWPage(document, window, true);
        });
    }
    async executeACT(page, sourceHtml, options) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/act-rules')
        });
        const metaElements = css_select_1.default('meta', sourceHtml.html.parsed);
        const parsedMetaElements = new Array();
        for (const element of metaElements || []) {
            const content = util_1.DomUtils.getSourceElementAttribute(element, 'content');
            const httpEquiv = util_1.DomUtils.getSourceElementAttribute(element, 'http-equiv');
            const htmlCode = util_1.DomUtils.getSourceElementHtmlCode(element, true, false);
            const selector = util_1.DomUtils.getSourceElementSelector(element);
            parsedMetaElements.push({
                content,
                httpEquiv,
                htmlCode,
                selector
            });
        }
        const actReport = await page.evaluate((parsedMetaElements, options) => {
            const act = new ACTRules.ACTRules(options);
            return act.execute(parsedMetaElements, window.page);
        }, parsedMetaElements, options);
        const r40 = 'QW-ACT-R40';
        if (!options || !options['rules'] || options['rules'].includes(r40) || options['rules'].includes('59br37')) {
            const viewport = page.viewport();
            await page.setViewport({
                width: 640,
                height: 512
            });
            const actReportR40 = await page.evaluate(() => {
                const act = new ACTRules.ACTRules();
                return act.executeQW_ACT_R40(window.page);
            });
            await page.setViewport({
                width: viewport.width,
                height: viewport.height
            });
            actReport.assertions[r40] = actReportR40;
            let outcome = actReportR40.metadata.outcome;
            if (outcome === "passed") {
                actReport.metadata.passed++;
            }
            else if (outcome === "failed") {
                actReport.metadata.failed++;
            }
            else if (outcome === "warning") {
                actReport.metadata.warning++;
            }
            else {
                actReport.metadata.inapplicable++;
            }
        }
        return actReport;
    }
    async executeWCAG(page, options, validation) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/wcag-techniques')
        });
        const url = page.url();
        const newTabWasOpen = await util_1.BrowserUtils.detectIfUnwantedTabWasOpened(page.browser(), url);
        const htmlReport = await page.evaluate((newTabWasOpen, validation, options) => {
            const html = new WCAGTechniques.WCAGTechniques(options);
            return html.execute(window.page, newTabWasOpen, validation);
        }, newTabWasOpen, validation, options);
        return htmlReport;
    }
    async executeBP(page, options) {
        await page.addScriptTag({
            path: require.resolve('@qualweb/best-practices')
        });
        const bpReport = await page.evaluate((options) => {
            const bp = new BestPractices.BestPractices();
            if (options)
                bp.configure(options);
            return bp.execute(window.page);
        }, options);
        return bpReport;
    }
    async evaluatePage(sourceHtml, page, execute, options, url, validation) {
        const evaluator = await this.getEvaluator(page, sourceHtml, url);
        const evaluation = new evaluationRecord_object_1.default(evaluator);
        await this.addQWPage(page);
        if (execute.act) {
            evaluation.addModuleEvaluation('act-rules', await this.executeACT(page, sourceHtml, options['act-rules']));
        }
        if (execute.wcag) {
            evaluation.addModuleEvaluation('wcag-techniques', await this.executeWCAG(page, options['wcag-techniques'], validation));
        }
        if (execute.bp) {
            evaluation.addModuleEvaluation('best-practices', await this.executeBP(page, options['best-practices']));
        }
        if (execute.wappalyzer) {
            evaluation.addModuleEvaluation('wappalyzer', await wappalyzer_1.executeWappalyzer(url));
        }
        return evaluation;
    }
    parseUrl(url, pageUrl) {
        const inputUrl = url;
        let protocol;
        let domainName;
        let domain;
        let uri;
        let completeUrl = pageUrl;
        protocol = completeUrl.split('://')[0];
        domainName = completeUrl.split('/')[2];
        const tmp = domainName.split('.');
        domain = tmp[tmp.length - 1];
        uri = completeUrl.split('.' + domain)[1];
        const parsedUrl = {
            inputUrl,
            protocol,
            domainName,
            domain,
            uri,
            completeUrl
        };
        return parsedUrl;
    }
}
exports.Evaluation = Evaluation;
